//*****************************************************************************
//
// qtmixer - an X windows soundcard mixer program.
// Copyright (C) 1997 - 2000  Tom Daley
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The license is in the file COPYING.
//
// Tom Daley
// 1309 W. Lorraine Ave.
// Woodland Park, CO  80863-2329
//
// daley@codenet.net (Tom Daley)
// http://users.codenet.net/daley
//
//*****************************************************************************

#include <math.h>

#include "stereo_slider.h"

//*****************************************************************************

QtedStereoSlider::
QtedStereoSlider(int left, int right, QWidget *parent, 
                 const char *name) : QWidget(parent, name) {
   if (left == right)		// if initial values are the same
      _locked = 1;		// make it a locked slider
   else
      _locked = 0;

   _mono = 0;			// default is stereo
   _left_p = new QtedSlider(1, 100, 5, 100 - left, QSlider::Vertical, this);
   _right_p = new QtedSlider(1, 100, 5, 100 - right, QSlider::Vertical, this);


// Propigate the right mouse button press
   connect(_left_p, SIGNAL(rightPressed()), this, SLOT(rightPressed()));
   connect(_right_p, SIGNAL(rightPressed()), this, SLOT(rightPressed()));

// when value of a slider changes, Update both if locked and propigate up 
   connect(_left_p, SIGNAL(valueChanged(int)),
           this, SLOT(setRawLockedValue(int)));
   connect(_right_p, SIGNAL(valueChanged(int)),
           this, SLOT(setRawLockedValue(int)));

   resizeEvent(NULL);
}

//*****************************************************************************

void QtedStereoSlider::
resizeEvent(QResizeEvent *ev_p) {
   int y = 0;
   int w = width(), h = height();
   int w_2 = w / 2;
   int w_4 = w / 4;

   if (!_locked) {
      _left_p->setGeometry(0, 0, w_2, h);
      _right_p->setGeometry(w_2, 0, w_2, h);
   } else {
      _left_p->setGeometry(w_4, 0, w_2, h);
      _right_p->setGeometry(w_4, 0, w_2, h);
   }
}

//*****************************************************************************

void QtedStereoSlider::
rightPressed() {
   if (! _mono) {
      if (_locked) {	// if locked
         _locked = 0;	// unlock
      } else {		// if locking, place sliders at the average of the two
         int val;
         val = (_left_p->value() + _right_p->value()) / 2;
         _locked = 1;
         _left_p->setValue(val);
         _right_p->setValue(val);
      }
      resizeEvent(NULL);
   }
}

//*****************************************************************************

void QtedStereoSlider::
setRawLockedValue(int val) {
   emit valueChanged();		// propigate signal
   if (_locked) {		// if sliders locked
      _left_p->setValue(val);	// set both to same value
      _right_p->setValue(val);
   }
}

//*****************************************************************************

void QtedStereoSlider::
setLeftValue(int val) {
   _left_p->setValue(100 - val);
}

//*****************************************************************************

int QtedStereoSlider::
leftValue(void) {
   return (100 - _left_p->value());
}

//*****************************************************************************

void QtedStereoSlider::
setRightValue(int val) {
   _right_p->setValue(100 - val);
}

//*****************************************************************************

int QtedStereoSlider::
rightValue(void) {
   return (100 - _right_p->value());
}

//*****************************************************************************

int QtedStereoSlider::
locked(void) {
   return (_locked);
}

//*****************************************************************************

void QtedStereoSlider::
setMono(int flag) {
   // if we are stereo, changing to mono, and slider not locked
   if (_mono == 0 && flag && !_locked) 
      rightPressed();			// toggle to locked
   _mono = flag;
}

//*****************************************************************************

int QtedStereoSlider::
mono(void) {
   return (_mono);
}

//*****************************************************************************
